/*******************************************************************************
* PURPOSE:       Check a dataset for duplicate values in key variable(s). Print
*                error (or warning or nothing) to log if duplicate found, also 
*                prints duplicate records to LST file.
*
* INPUT:         Input parameters listed below    
*
*                DATA:   Input dataset - REQUIRED
*                BYVARS: Key (Variables BY which you search for duplicates) - REQUIRED
*                OUTDATA: Optional output dataset that only contains the 
*                  duplicates
*                ADDVARS: Additional variables to print.
*                PRINT: Print duplicate records (default=Y)
*                TITLE2: title2 text printed above duplicates.  Only used if 
*                  PRINT=Y.  The title is fixed by the program and cannot be
*                  changed.
*                FMTBY: Format the by variables when printing? (yes/no, default=Y)
*                IFDUP: Defines behavior if a duplicate was found. This does not
*                  affect printing of duplicates. This can be ERR/WARN/ENDSAS/NOTHING
*                * ERR: Print error to log (default).
*                * WARN: Print warning to log.
*                * NOTE: Print note to log.
*                * ENDSAS: Print error to log and end SAS
*                * NOTHING: Do nothing.
* OUTPUT:        Output dataset as specified in OUTDATA parameter, warning or 
*                error to log file, printout of duplicates to LST. 
* MACROS USED:   yesnosub, varexst, alrdexst
* NOTES:         NA 
* MODIFICATIONS:  
*******************************************************************************/

%macro dupkey(DATA=, OUTDATA=, BYVARS=, ADDVARS=, TITLE2=, PRINT=Y, FMTBY=Y, IFDUP=err);

/***************************;
* Validate input parameters ;
****************************/
%local ALREADYX INTERNAL_DATASETS NOBS BYVARCOMMA BYVARNODESC THISVAR i;
%let INTERNAL_DATASETS=__dupkey_tmp1 __dupkey_tmp2 __dupkey_tmp3;
%let alreadyx=%alrdexst(&INTERNAL_DATASETS);
%let ifdup=%upcase(&IFDUP);
%if %length(&ALREADYX)>0 %then %do;
  %put %upcase(error): Dataset(s) &ALREADYX already exist(s)!  These sets are used in DUPKEY macro;
%end;

%if %length(%alrdexst(&DATA))=0 %then %do;
  %put %upcase(error): Dataset in DATA parameter (&DATA) does not exist;
  %goto exit;
%end;

/*****************************************************************
* Remove the words DESC or DESCENDING from BYVARS variable
*****************************************************************/
%let i=1;
%let THISVAR=%scan(&BYVARS, &i, %str( )); 
%do %while (%length(&THISVAR)>0);
  %if %upcase(&THISVAR) ne DESC and %upcase(&THISVAR) ne DESCENDING %then %do;
    %let BYVARNODESC=&BYVARNODESC &THISVAR;
  %end;
  %let i=%eval(&i+1);
  %let THISVAR=%scan(&BYVARS, &i, %str( )); 
%end;

%varexst(data=&DATA, vars=&BYVARNODESC, addmsg=%str( (specified in BYVARS parameter)), iferr=endsas);

%yesnosub(print fmtby);

%if %length(&DATA)=0 %then %put %upcase(error): Dupkey macro.  DATA parameter is required;
%if %length(&BYVARS)=0 %then %put %upcase(error): Dupkey macro.  BYVARS parameter is required;
%if &IFDUP ne WARN and &IFDUP ne ERR and &IFDUP ne ENDSAS and &IFDUP ne NOTHING and &IFDUP ne NOTE %then %do;
  %put %upcase(error): DUPKEY macro.  IFDUP parameter (&IFDUP) should be one of: WARN, ERR, NOTE, ENDSAS, NOTHING;
  %goto exit;
%end;
%if &PRINT ne Y and &PRINT ne N %then %do;
  %put %upcase(error): DUPKEY macro.  PRINT parameter (&PRINT) should be N, Y, NO, YES;
  %goto exit;
%end;
%if &FMTBY ne Y and &FMTBY ne N %then %do;
  %put %upcase(error): DUPKEY macro.  FMTBY parameter (&FMTBY) should be N, Y, NO, YES;
  %goto exit;
%end;

/**********************;
* Identify duplicates *;
***********************/
proc sort data=&DATA out=__dupkey_tmp1;
  by &BYVARS;
run;

proc summary data=__dupkey_tmp1;
  by &BYVARS;
  output out=__dupkey_tmp2 (drop=_TYPE_);
  format _all_;
run;

proc sql noprint;
  select count(*) into :NOBS from __dupkey_tmp2 where _FREQ_>1;
quit;
/* Convenient way to drop compress without relying on macro funtion */
%let NOBS=&NOBS;

/****************************************************************;
* Moved this here because we might want the output dataset 
*  even if we arent printing it. (This used to be inside the 
*  PRINT=Y if statement below.)  Ray G, May, 2011;
*****************************************************************/
data __dupkey_tmp3 (drop=_FREQ_);
  merge __dupkey_tmp1 (in=a) __dupkey_tmp2 (in=b where=(_FREQ_>1));
  by &BYVARS;
  if a and b;
run;

/********************************;
* Print duplicates, if requested ;
*********************************/
%if &PRINT=Y and &NOBS>0 %then %do;
title "DUPKEY MACRO: Duplicate values in set (&DATA) using key (&BYVARS)";
%if %length(&TITLE2)>0 %then %do; 
  title2 &TITLE2; 
%end;

/*************************************************;
* If ADDVARS=_all_, remove the variables in BYVARS;
*   from the list of variables for printing       ;
**************************************************/
%let BYVARCOMMA=%sysfunc(tranwrd("%upcase(&BYVARNODESC)", %str( ), ","));
%if %upcase(&ADDVARS)=_ALL_ %then %do;
  proc sql noprint;  
    select NAME into :ADDVARS separated by " "
    from DICTIONARY.COLUMNS where LIBNAME="WORK" 
    and upcase(MEMNAME)="__DUPKEY_TMP1" and upcase(name) not in (&BYVARCOMMA)
    order by npos; 
  quit;
%end;

proc print data=__dupkey_tmp3;
  var &ADDVARS;
  by &BYVARS;
  id &BYVARNODESC;
%if &FMTBY=N %then %do;
  format &BYVARNODESC;
%end;
run;
title;
%end;

%if %length(&OUTDATA)>0 %then %do;
data &OUTDATA;
  set __dupkey_tmp3;
run;
%end;

/****************************************************;
* Print error or warning, and exit SAS, if requested ;
*****************************************************/
%if &NOBS>0 %then %do;
  %if &IFDUP=WARN %then %do;
    %put %upcase(warning): &NOBS duplicate records in &DATA using key (&BYVARS);
  %end;
  /* [CS20141211] */
  %else %if &IFDUP=NOTE %then %do;
    %put %upcase(note: (CBAR)) &NOBS duplicate records in &DATA using key (&BYVARS); 
  %end;  
  %else %if &IFDUP=ERR or &IFDUP=ENDSAS %then %do; 
    %put %upcase(error): &NOBS duplicate records in &DATA using key (&BYVARS);
  %end;
  %if &IFDUP=ENDSAS %then endsas;;
%end;

%exit:

proc datasets nolist;
  delete &INTERNAL_DATASETS;
run; quit;

%mend dupkey;

