/******************************************************************************/
/* PURPOSE:       Create a report table for counts of AE (adverse events) or 
/*			 a similar type table from FLAGSUM-like output.
/* 
/* INPUT:         Dataset listed in INDATA
/*		  REQUIRED VARIABLES:
/*			&GRPEVTVAR 	- name of variable holding group and event 
/*					  info is entered, 
/*			&COLVAR 	- name of variable holding column/treatment 
/*					  info is entered, 
/*			rptme 		- name of variable likely created in FLAGSUM
/*					  that contains numbers/pcts to print in table
/*
/* OUTPUT:        Produce PROC REPORT code to be placed with ODS statements.
/*
/* INTERNAL MACROS WITH THEIR DESCRIPTION, GOOD TO MAKE IF THEY DO NOT ALREADY EXIST 
/* MACROS USED:   %dataexst - check dataset exists
/*                %varexst  - check vars exist in dataset
/*                %nobs     - retrieve number of obs in a dataset
/* 
/******************************************************************************/

%macro flagrpt( INDATA     =
	      , GRPVAR	   = rptgrp
	      , EVTVAR	   = rptevt
	      , GRPEVTVAR  = grpev1col
	      , COLVAR     = rptcol
	      , SUBVAR     =

	      , GRPEV1COLW =
	      , PCTMODE    = 1
	      , PCTFMT	   =
	      , SUMMARY    = N

	      , EVTTITLE   = Adverse Events
	      , COLTITLE   = 
	      , SUBTITLE   =

              , RTFFNTSZ   =

              , DEBUG  = N
              );

%local  USERLS
	CLIST _C_MAX 
	A B C D E F
	ZEROLIST NEXT
	GRPEVFTW
	;


/**************/
/* BEGIN CODE */
/**************/
/* CHECK THAT THERE IS A VALUE IN INDATA */
%if %length(&INDATA)= 0 %then %do;
    %put %upcase(error:) INDATA requires a value.; 
    %put %upcase(error:) FLAGRPT will now terminate.;
    %abort abend 1;
%end;

/* CHECK INDATA DATASET EXISTS */
%dataexst(&INDATA);

/* CHECK THAT THE DATASET HAS OBSERVATIONS */ 
%if %nobs(&INDATA)= 0 %then %do;
    %put %upcase(note: (cbar)) &INDATA does not contain any observations.;
    %put %upcase(note: (cbar)) Please check dataset and submit it with observations.;
    %put %upcase(note: (cbar)) FLAGRPT will now return to the SAS program.;
    %return;
%end;

/********************************************************/
/* ADD DEBUG OPTION TO PARAMETERS                       */
/********************************************************/
%if %length(&DEBUG)= 0 %then %let DEBUG = Y;
%else                        %let DEBUG = %substr(%upcase(&DEBUG),1,1);

%if &DEBUG = Y %then %do;
   options mprint mlogic mlogicnest source2 symbolgen;
%end;

/***************************************************/
/* CHECK THAT SOME VALUE IS IN REQUIRED MACRO VARS */
/***************************************************/

%if %length(&GRPEVTVAR) = 0 or  %length(&COLVAR) = 0  %then %do; 

   %put %upcase(note: (cbar)) Either GRPEVTVAR or COLVAR is missing.;
   %put %upcase(note: (cbar)) The defaults are grpev1col and rptcol, respectively.;
   %put %upcase(note: (cbar)) FLAGRPT will continue with those values.;
   %put %upcase(note: (cbar)) If this is not correct, please change these parameters and rerun.;
   %put %upcase(note: (cbar)) If this is correct and want to avoid this message, ;
   %put %upcase(note: (cbar))    please delete these parameters and rerun.;
   
%end;

/* SINCE USING DEFAULTS FOR GRPEVTVAR AND COLVAR IF MISSING */
/* THEN NEED TO SET THESE VALUES BEFORE USING %varexst()    */
/********************************************************/
/* SET DEFAULT FOR GRPVAR    IF BLANK			*/
/* SET DEFAULT FOR EVTVAR    IF BLANK			*/
/* SET DEFAULT FOR GPREVTVAR IF BLANK                   */
/* SET DEFAULT FOR COLVAR    IF BLANK                   */
/* SET DEFAULT FOR EVTTITLE  IF BLANK                   */
/* SET DEFAULT FOR COLTOTVAL AND SUBTOTVAL IF BLANK	*/
/*	BOTH VARS ARE GLOBAL FROM FLAGSUM		*/
/*	NEED TO CHECK IF THEY EXIST FROM FLAGSUM	*/
/* SET DEFAULT FOR PCTMODE   IF BLANK                   */
/*          ADJUST PCTMODE AS NEEDED                    */
/* SET DEFAULT FOR PCTFMT  IF BLANK                     */
/*           CHECK PCTFMT  EXISTS                       */
/********************************************************/
%if %length(&GRPVAR)	= 0 %then %let GRPVAR	 = rptgrp;
%if %length(&EVTVAR)	= 0 %then %let EVTVAR	 = rptevt;
%if %length(&GRPEVTVAR) = 0 %then %let GRPEVTVAR = grpev1col;
%if %length(&COLVAR)    = 0 %then %let COLVAR    = rptcol;
%if %length(&EVTTITLE)  = 0 %then %let EVTTTILE  = Adverse Events;
%if %length(&SUMMARY)	= 0 %then %let SUMMARY	 = N;
%else				  %let SUMMARY   = %substr(%upcase(&SUMMARY),1,1);

/* NEEDED FOR CODING 0 VALUES WHEN PCTMODE = 4-7 and 10 */
%if %symexist(COLTOTVAL) = 0 %then %do;
      %local COLTOTVAL;
      %let   COLTOTVAL = TOTAL;
   %if %length(&SUBVAR) > 0 %then %do;
      %local SUBTOTVAL;
      %let   SUBTOTVAL = All;
   %end;
%end;

options minoperator;
%if %length(&PCTMODE) = 0 %then %let PCTMODE = 1;
%if ^(&PCTMODE in 0 1 2 3 4 5 6 7 8 9 10) 
                          %then %let PCTMODE = 1;

/* FYI - CANNOT CHECK EXISTENCE OF FORMATS LIKE 4. OR 7.2 */
/*       AND THESE ARE EXPECTED TO BE USED IN PCTFMT      */
/*       SO THERE IS NO CHECK FOR EXISTENCE OF FMT        */
/* flgsmft IS THE DEFAULT IN FLAGSUM AS WELL		  */
/**********************************************************/
%if %length(&PCTFMT) = 0 %then %do;
   proc format;
        value flgsmft
             0<-<.5 = "<0.5"
             other  = [4.]
             ;
   run;
   %let PCTFMT = flgsmft4.;
%end;
%else %do;
   %if %sysfunc(find(&PCTFMT,%str(.))) = 0 %then %do;
      %let PCTFMT = %sysfunc(strip(&PCTFMT%str(.)));
   %end;
   %put PCTFMT: &PCTFMT;
%end;

/* DELETE PUNCTUATION AND DIGITS FROM PCTFMT VALUE  */
/* THIS GIVES NAME OF FORMAT TO SEARCH IF IT EXISTS */
%let CHKFMT = %sysfunc(compress(&PCTFMT,,dp));

/* CHECK FORMAT IF NOT SOLELY A NUMERIC FORMAT, LIKE 7. or 7.2 */
%if %length(&CHKFMT) ^= 0 %then %do;
   proc sql noprint;
      create table chkflagfmt
      as select libname, fmtname, source, fmttype
      from dictionary.formats
      where fmtname="%upcase(&CHKFMT)"
      ;
  quit;

  %if %nobs(chkflagfmt) = 0 %then %do;
     %put %upcase(error:) The following format does not exist:  &PCTFMT.; 
     %put %upcase(error:) Please enter a format that exists in  PCTFMT.; 
     %put %upcase(error:) Once corrections are made, please try running FLAGRPT again.;
     %put %upcase(error:) FLAGRPT will now terminate.;
     %abort abend 2;
  %end;
%end;

/****************************************/
/* CHECK THAT VARIABLES EXIST           */
/****************************************/
%varexst(DATA   = &INDATA., 
         VARS   = &GRPVAR &EVTVAR &GRPEVTVAR &COLVAR &SUBVAR, 
         ADDMSG = %str(for FLAGRPT.  Please include this variable in &INDATA and rerun the program.),
         IFERR  = endsas);

/**********************************/
/* ENSURE LINESIZE IS LONG ENOUGH */
/* FOR PRINTING BY SETTING ls=max */
/**********************************/
proc sql noprint;
     select setting into: USERLS 
     from sashelp.voption
     where upcase(optname)='LINESIZE';
quit;

options ls = max;

/**********************************************************/
/* CHANGE MOST PCTMODE CODES TO ONLY 4 values: 0, 1, 2, 3 */
/* VALUES > 3 NEED TO CHANGE 				  */
/* FOR THE OUTPUT INTENDED BY THESE VALUES THE USER	  */
/* LIKELY NEEDS TO CREATE A DATASET FIRST AND THEN	  */
/* PROC REPORT IT, 					  */
/* THESE CHANGED VALUES JUST HELP US GET THRU FLAGRPT 	  */
/* THESE PCTMODE VALUES CANNOT BE CODED CORRECTLY B/C	  */
/* DO NOT KNOW WHICH _c?_ VARS CONTAIN VALUES TO CODE 	  */
/* THESE OUTPUTS					  */
/**********************************************************/
%if %length(&SUBVAR) = 0 %then %do;
   %if 	     &PCTMODE =  6    %then %let PCTMODE = 1;
   %else %if &PCTMODE =  7 or 
       	     &PCTMODE = 10    %then %let PCTMODE = 3;; 
%end;

   %if 	     &PCTMODE =  8    %then %let PCTMODE = 2;
   %else %if &PCTMODE =  9    %then %let PCTMODE = 0;;


/************************************/
/* CALCULATE NUMBER OF _C?_ COLUMNS */
/* FOR 0 VALUES IN TABLE	    */
/************************************/
/*********************************************/
/* GET LIST OF FORMATTED VALUES OF COLVAR    */
/* FOR NOW: ONLY NON-MISSING VALUES INCLUDED */
/*********************************************/
/* PROC SQL HAS FEEDBACK OPTION THAT SHOWS MORE ABOUT WHAT IT IS DOING 	*/
/* LEFT IT IN BUT COMMENTED OUT AS A REMINDER IF NEEDED FOR DEBUGGING  	*/ 
/************************************************************************/
proc sql noprint; * feedback;
   select distinct &COLVAR into : CLIST separated by '@' 
   from &INDATA
   where ^(missing(&COLVAR))  
   ;
quit;

/* SET _C_MAX FROM NUMBER OF ITEMS IN CLIST */
%let _C_MAX = %sysfunc(countw(%nrbquote(&CLIST),@));

/* GET VALUE WITH COLTOTVAL */
/* ONLY NEED TOTAL INFO IF PCTMODE > 3 */
%if &PCTMODE > 3 %then %do;
   %do A=1 %to &_C_MAX;
      %if %index(%qscan(%nrbquote(&CLIST), &A, @), &COLTOTVAL) > 0 %then %do;
         %local COLTOTNUM;
         %let COLTOTNUM  = &A;
	 %local CTOTLIST CTOTUP;
/* Adjusted CTOTUP from +1 TO +3 FOR ADDITION OF rptgrp AND rptevt */
	 %let CTOTUP = %eval(&A+3);
	 %let CTOTLIST = _c&CTOTUP._;
      %end; 
   %end;
   %if %symexist(CTOTLIST) = 0 %then %do;
/* CHANGE PCTMODE THAT DEAL WITH TOTITLE WHEN NO TOTITLE FOUND */
        %if &PCTMODE = 4 %then %let PCTMODE = 1;;
        %if &PCTMODE = 5 %then %let PCTMODE = 3;;
   %end;
%end;


/*********************************************/
/* GET LIST OF FORMATTED VALUES OF COLVAR    */
/* FOR NOW: ONLY NON-MISSING VALUES INCLUDED */
/*********************************************/
/* PROC SQL HAS FEEDBACK OPTION THAT SHOWS MORE ABOUT WHAT IT IS DOING */
/***********************************************************************/
%if %length(&SUBVAR) > 0 %then %do;
   %local SLIST;
   proc sql noprint; * feedback;
      select distinct &SUBVAR into : SLIST separated by '@' 
      from &INDATA
      where ^(missing(&SUBVAR)) 
      ;
   quit;

/*********************************************************/
/* SET _C_MAX FROM NUMBER OF ITEMS IN CLIST 		 */
/* MULTIPLY PREV _C_MAX VALUE BY NUMBER OF SUBVAR LEVELS */
/* THIS GETS NUMBER OF _C?_ COLUMNS FOR ZEROS ARRAY 	 */
/*********************************************************/
   %let _C_MAX = %eval(%eval(%sysfunc(countw(%nrbquote(&SLIST),@)))*&_C_MAX);

/* GET VALUE WITH SUBTOTVAL */
/* ONLY NEED TOTAL INFO IF PCTMODE > 3 */
   %if &PCTMODE > 3 %then %do;
      %local _S_MAX;
      %let _S_MAX = %sysfunc(countw(%nrbquote(&SLIST),@));
      %local B;
      %do B=1 %to &_S_MAX;
         %if %index(%qscan(%nrbquote(&SLIST), &B, @), &SUBTOTVAL) > 0 %then %do; 
	    %local SUBTOTNUM STOTLIST SUPONE;;
	    %let STOTLIST  =;
/* GET LIST OF COLTOTITLE COLUMNS */
/* Adjusted SUPONE from +1 TO +3 FOR ADDITION OF RPTGRP AND RPTEVT */
            %do C= &B %to &_C_MAX %by &_S_MAX;
		%let SUPONE = %eval(&C+3);
                %let STOTLIST = &STOTLIST _c&SUPONE._;
	    %end;
         %end;
      %end;
/* GET LIST OF TOTITLE COLUMNS */
      %if &PCTMODE = 4 or &PCTMODE = 5 %then %do;
          %local NOW END;

/* CALCULATIONS BELOW FOR rptgrp AND rptevt */
/*		  	ADDED TO PROC REPORT FROM INDATA    */
/* +4 BECAUSE FIRST COLUMN IS _c4_ 	*/
/* +3 TO ACCOUNT FOR grpev1col, rptgrp, AND rptevt COLUMNS 	*/
          %let NOW = %eval(((&COLTOTNUM-1)*&_S_MAX)+4);
          %let END = %eval((&COLTOTNUM*&_S_MAX)+3);

/* CLEAR CTOTLIST OF PREVIOUS VALUE */
          %let CTOTLIST =;
/* MAKE LIST FOR TOTITLE COLUMNS */
          %do D = &NOW %to &END;
             %let CTOTLIST = &CTOTLIST _c&D._;
          %end;
      %end;
      %if %symexist(STOTLIST) = 0 %then %do;
/* CHANGE PCTMODE THAT DEAL WITH COLTOTITLE WHEN NO COLTOTITLE FOUND */
           %if &PCTMODE = 6  or
               &PCTMODE = 10 %then %let PCTMODE = 1;;
           %if &PCTMODE = 7  %then %let PCTMODE = 3;;
      %end;
   %end;
%end;

/*****************************************************************/
/* MAKE LIST OF _c?_ VARS FOR ZEROS ARRAY IN COMPUTE BLOCK BELOW */
/*****************************************************************/
/* NEED TO CHANGE +1 TO +3 BECAUSE NOW INCLUDE 	                 */
/*	RPTGRP AND RPTEVT VARIABLES INTO PROC REPORT		 */
/*****************************************************************/
%let ZEROLIST = ;
%do E=1 %to &_C_MAX;
   %let NEXT = %eval(&E+3);
   %let ZEROLIST = &ZEROLIST _c&NEXT._;
%end;

/* FIND WIDTH TO USE FOR grpev1col 	*/
/* IF SUMMARY = Y, DELETE INDENTED RECS */
/* AKA EVENT RECORDS			*/
/****************************************/
data _null_;
   set &INDATA;
   call symput("GRPEVFTW",strip(vformatw(&GRPEVTVAR)));
%if &SUMMARY = Y %then %do;
   if prxmatch('/\s/', grpev1col) = 1 then delete;      
%end;

run;

/***************************/
/* FINAL TABLE PROC REPORT */
/***************************/
/************************************************/
/* nocompletecols CREATES CONSISTENT COLUMNS 	*/
/* WHETHER VALUES WERE FOUND OR NOT FOR THOSE 	*/
/* VALUES					*/
/************************************************/

proc report nowd data= &INDATA 
%if &SUMMARY = Y %then %do;
   (where=(prxmatch('/\s/', grpev1col) ^= 1))
%end;
		 out = flagrptd 
	    split='$' missing nocompletecols

/* IF PROBLEMS, CAN ADD style(calldef lines) TO BELOW */
/******************************************************/
    %if %length(&RTFFNTSZ) > 0 %then %do;
       style(column header report summary)=[font_size=&RTFFNTSZ pt]
    %end;
    ;
   columns ("&EVTTITLE" &GRPVAR &EVTVAR &GRPEVTVAR) &COLVAR, 
   %if %length(&SUBVAR) > 0 %then %do;
	    &SUBVAR, 
   %end;
	    rptme dummy;

      define  &GRPVAR	 / group order= data noprint;
      define  &EVTVAR    / group 	     noprint;
      define  &GRPEVTVAR / group " "   
   %if %length(&GRPEV1COLW)>0 %then %do;
           style=[cellwidth=&GRPEV1COLW];
   %end;
   %else %do;
           style=[cellwidth=%eval(7*(&GRPEVFTW)/12) em];
   %end;
    ;	

      define  &COLVAR    / across  "&COLTITLE " order= internal;
   %if %length(&SUBVAR) > 0 %then %do;
      define  &SUBVAR    / across  "&SUBTITLE " order= internal;
   %end;
      define  rptme      / display " " center;
      define  dummy      / noprint;
      compute dummy;

/* REMINDER */
/* PCTMODE = 8 CHANGED TO 2 */
/* PCTMODE = 9 CHANGED TO 0 */

      %if &PCTMODE < 4 %then %do;
        array zeros &ZEROLIST;
        do z=1 to dim(zeros);
           if missing(zeros(z)) then do;

        %if &PCTMODE = 1 %then %do;
              if prxmatch('/\s/', &GRPEVTVAR) ^= 1 then do;
                 zeros(z) = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
              end;
              else do;
                 zeros(z) = put(0,3. -r);
              end;
        %end;
        %else %if &PCTMODE = 0 %then %do;
                 zeros(z) = put(0,3. -r);
        %end;
        %else %if &PCTMODE = 2 %then %do;
                 zeros(z) = '('||strip(put(0,&PCTFMT))||'%)';
        %end;
        %else %if &PCTMODE = 3 %then %do;
                 zeros(z) = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
        %end;
           end;
        end;
      %end;

/* IF PCTMODE >= 4 */
/* REMINDER */
/* PCTMODE = 8 CHANGED TO 2 */
/* PCTMODE = 9 CHANGED TO 0 */

      %else %do;
         %do F=1 %to &_C_MAX;
            %local THIS;
            %let   THIS = %sysfunc(strip(%scan(&ZEROLIST, &F,' ')));

/* USE CTOTLIST FOR INDEX WHEN 4 OR 5*/
            %if &PCTMODE = 4 %then %do;
               %if %index(&CTOTLIST, &THIS) > 0 %then %do;
                    if missing(&THIS) then do;
                        if prxmatch('/\s/', &GRPEVTVAR) ^= 1 then do;
                            &THIS = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
                        end;
                        else do;
                            &THIS = put(0,3. -r);
                        end;
                    end;
               %end;
               %else %do;
                    if missing(&THIS) then do;
                            &THIS = put(0,3. -r);
                    end;
               %end;
            %end;
            %if &PCTMODE = 5 %then %do;
               %if %index(&CTOTLIST, &THIS) > 0 %then %do;
                    if missing(&THIS) then do;
                            &THIS = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
                    end;
               %end;
               %else %do;
                    if missing(&THIS) then do;
                            &THIS = put(0,3. -r);
                    end;
               %end;
            %end;

/* USE STOTLIST FOR INDEX WHEN 6, 7, OR 10 */
            %if &PCTMODE = 6 %then %do;
               %if %index(&STOTLIST, &THIS) > 0 %then %do;
                    if missing(&THIS) then do;
                        if prxmatch('/\s/', &GRPEVTVAR) ^= 1 then do;
                            &THIS = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
                        end;
                        else do;
                            &THIS = put(0,3. -r);
                        end;
                    end;
               %end;
               %else %do;
                    if missing(&THIS) then do;
                            &THIS = put(0,3. -r);
                    end;
               %end;
            %end;
            %if &PCTMODE = 7 %then %do;
               %if %index(&STOTLIST, &THIS) > 0 %then %do;
                    if missing(&THIS) then do;
                            &THIS = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
                    end;
               %end;
               %else %do;
                    if missing(&THIS) then do;
                            &THIS = put(0,3. -r);
                    end;
               %end;
            %end;
            %if &PCTMODE = 10 %then %do;
               %if %index(&STOTLIST, &THIS) > 0 %then %do;
                    if missing(&THIS) then do;
                            &THIS = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
                    end;
               %end;
               %else %do;
                    if missing(&THIS) then do;
                        if prxmatch('/\s/', &GRPEVTVAR) ^= 1 then do;
                            &THIS = put(0,3. -r)||' '||'('||strip(put(0,&PCTFMT))||'%)';
                        end;
                        else do;
                            &THIS = put(0,3. -r);
                        end;
                    end;
               %end;
            %end;
         %end; /* FOR F DO LOOP */
      %end;    /* FOR ELSE DO OF PCTMODE >= 4 */

         endcomp;
/* GET INDENT OF &GRPEVTVAR EVENT ROWS */
         compute &GRPEVTVAR;
         %if &SUMMARY = Y %then %do;
	      call define(_col_,"style","style=[just=l]");
	      if strip(&GRPEVTVAR)= "Overall" then do;
                 call define(_row_,"style","style=[vjust=bottom font_weight=bold]");
              end;
         %end;
         %else %do;
              if prxmatch('/\s/', &GRPEVTVAR) ^= 1 then do;
                 call define(_col_,"style","style=[just=l]");
	         call define(_row_,"style","style=[vjust=bottom font_weight=bold]");
              end;
              else do;
                 call define(_col_,"style","style=[leftmargin=4 em]");
              end;
         %end;
         endcomp;
run;

/* RETURN LINESIZE TO USER SPECIFICATION */
options ls = &USERLS;

%mend  flagrpt;


