/***********************************************************/
/* PURPOSE:       Create a dataset with counts intended for an AE (adverse events) or 
/*			similar type table. 
/*		  INCLUDED: 1 var of percents, 1 of counts and pcts together, 
/*			    vars for denoms of each COLVAR value (likely treatment values)
/*
/* INPUT:         2 datasets: 
/*			1. EVTDATA in occurrence data structure AND containing all variables
/*				listed in any of the parameters, including any
/*				population flag variables from COLDATA (likely ADSL) 
/*				listed in COLWHERE that are used for the analysis - 
/*				aka counting of flag variables.
/*			2. COLDATA in an ADSL-like data structure and solely used
/*				to count denominator values for calculations in the
/*				macro.  It will NOT be merged onto EVTDAT.
/*				It must contain all variables listed in COLWHERE.
/*
/* OUTPUT:        LONG (not WIDE) flagsum dataset that can be reported with a simple 
/*                      PROC REPORT
/*
/* INTERNAL MACROS WITH THEIR DESCRIPTION, GOOD TO MAKE IF THEY DO NOT ALREADY EXIST
/* MACROS USED:   %dataexst - check dataset exists
/*		  %paramreq - check have required parameters
/*		  %varexst  - check vars exist in dataset
/*		  %nobs     - retrieve number of obs in a dataset
/*		  %dupkey   - check for duplicate records in dataset 
/*
/************************************************************/

%macro flagsum(	OUTDATA  =,
		COLDATA  =, 
		COLWHERE =, 
		COLVAR   =, 
		EVTDATA  =,
		GRPVAR   =,
		EVTVAR   =,
		SUBVAR   =, 
	        IDVAR	 = usubjid,
		ALLFLG   =, 
		GRPFLG   =, 
		EVTFLG   =, 
		PCTMODE  = 1, 
                PCTFMT   =, 
	        LSTMORE  = N, 
		DEBUG    = N
	      ); 
		
/* DECLARE LOCAL MACRO VARS SO NOT TO OVERRIDE USER MACRO VARS */
%local 	   A B C D E F 
	   USERMISS NOOBIE ATLEAST1FLG 
	   _NEWAND CHKFMT
	   DSID1 DSID2 
	   COLVTYPE1 COLVTYPE2 FMTEQ COLFMT1 COLFMT2 COLTOTVAL
	   IDEVTYPE IDEVLEN 
	   IDCVTYPE IDCVLEN
	   IDLEN FLGIDLEN RC RC2 
	   MISSNUM FLGLIST FLGLISTMAX FLGLISTWH
	   CVNMTOP GRPEVFTW RPTCOLW
	   CLIST CLISTMAX 
	   OOPSNUM MANYGRPS
	   SETALL SETGRP SETEVT
	   DENOMTOTAL DENOMLIST 
	   RPTMEW
	;

/* STORE missing SYSTEM OPTION FOR USER */
/*   SET missing = . 			*/
/* ONLY USED IN PROCs 			*/
         
%let USERMISS = %sysfunc(GETOPTION(missing));
options missing = .;

/* CHECK THAT THERE IS A VALUE IN EVTDATA AND COLDATA */
%if %length(&EVTDATA)= 0 or %length(&COLDATA)= 0 %then %do;
   %if %length(&EVTDATA)= 0 %then %do;
       %put %upcase(error:) EVTDATA requires a value and currently there is none.;
       %put %upcase(error:) Please enter a value into EVTDATA.; 
   %end;
   %if %length(&COLDATA)= 0 %then %do;
       %put %upcase(error:) COLDATA requires a value and currently there is none.;
       %put %upcase(error:) Please enter a value into COLDATA.; 
   %end;
       %put %upcase(error:) Once corrections are made, please try running FLAGSUM again.;
       %put %upcase(error:) FLAGSUM will now terminate.;
       %abort abend 1;
%end;

/* CHECK BOTH THAT DATASETS EXIST */
%dataexst(&EVTDATA &COLDATA);

/* CHECK THAT THE DATASETS HAVE OBSERVATIONS */ 
%if %nobs(&EVTDATA)= 0 or %nobs(&COLDATA)= 0 %then %do;
   %if %nobs(&EVTDATA)= 0 %then %do;
         %put %upcase(error:) &EVTDATA does not have any observations.;
   %end;
   %if %nobs(&COLDATA)= 0 %then %do;
         %put %upcase(error:) &COLDATA does not have any observations.;
   %end;
         %put %upcase(error:) Please check datasets and submit them with observations.;
         %put %upcase(error:) FLAGSUM will now terminate.;
         %abort abend 2;
%end;

/* CHECK OF OBSERVATIONS WITH COLWHERE VALUE */
%if %length(&COLWHERE) > 0 %then %do;
  
/* When NOOBIE = 1 THEN IT MEANS THERE WAS AN ERROR */
   %let NOOBIE = ;

   %if %nobs(&EVTDATA,&COLWHERE) = 0 %then %do;
            %put %upcase(error:) ----EVTDATA------------------------------------------------------------------------------------;
            %put %upcase(error:) The &EVTDATA data with the COLWHERE parameter does not have any observations.			;
            %put %upcase(error:) Here is the COLWHERE value for &EVTDATA: 							;
            %put %upcase(error:)      &COLWHERE 										;
            %put %upcase(error:) Please check there are still observations in &EVTDATA when the COLWHERE parameter is used.	;
            %let NOOBIE = 1;
   %end;

/* THE LINE WITH JUST &COLWHERE IS INTENTIONALLY OFF BY 1 OF ABOVE SO PRINTED OUT BY chk PROGRAM 	*/
/* OTHERWISE chk DELETES ALL DUPLICATE LINES AND THIS WOULD BE DELETED IF BOTH DATASETS HAVE THE PROBLEM */
   %if %nobs(&COLDATA,&COLWHERE) = 0 %then %do;
	    %put %upcase(error:) ----COLDATA------------------------------------------------------------------------------------;
            %put %upcase(error:) The &COLDATA data with the COLWHERE parameter does not have any observations.			;
	    %put %upcase(error:) Here is the COLWHERE value for &COLDATA: 							;
 	    %put %upcase(error:)     &COLWHERE 											;
            %put %upcase(error:) Please check there are still observations in &COLDATA when the COLWHERE parameter is used.	;
            %let NOOBIE = 1;
   %end;

/* ABORT IF ONE OF ERRORS OCCURRED */
   %if &NOOBIE = 1 %then %do;
            %put %upcase(error:) -----------------------------------------------------------------------------------------------;
            %put %upcase(error:) FLAGSUM will now terminate.                                                                    ;
	    %abort abend 3;
   %end;

    	    %put %upcase(note: (cbar)) It looks like we are using a COLWHERE value. 				;
            %put %upcase(note: (cbar)) COLWHERE is only intended to be used with population flag variables.     ;
            %put %upcase(note: (cbar)) Problems in the COLWHERE value can cause FLAGSUM to fail painfully.	;
            %put %upcase(note: (cbar)) BE CAREFUL.....								;
%end;

/***************************************************/
/* CHECK THAT SOME VALUE IS IN REQUIRED PARAMETERS */
/***************************************************/
%if %length(&GRPVAR)  = 0 or  %length(&EVTVAR) = 0 or
    %length(&COLVAR)  = 0 %then %do; 

   %paramreq(GRPVAR EVTVAR COLVAR);

   %if %length(&SUBVAR) > 0 and %length(&COLVAR) = 0 %then %do;
      %put %upcase(error:) %str( );
      %put %upcase(error:) COLVAR is not defined and SUBVAR is.;
      %put %upcase(error:) If you want to use SUBVAR, a definition for COLVAR is required.;
      %put %upcase(error:) Perhaps consider creating a dummy variable to enter into COLVAR.;
   %end;

   %put %upcase(error:) Please define the above parameters and try again.;
   %abort abend 4;

%end;

/************************************************/
/* CHECK THAT AT LEAST 1 FLAG PARAM HAS A VALUE */
/************************************************/
%let ATLEAST1FLG = &ALLFLG &GRPFLG &EVTFLG;
%if %length(&ATLEAST1FLG) = 0 %then %do;
      %put %upcase(error:) At least 1 FLG parameter must have a value.;
      %put %upcase(error:) Here are the FLG parameters:  ;
      %put %upcase(error:) 	ALLFLG GRPFLG EVTFLG	 ;
      %put %upcase(error:) Please enter a value for at least one FLG parameter ; 
      %put %upcase(error:) and rerun the program.;
      %abort abend 5;
%end;

/********************************************************/
/* CHECK IF IDVAR ENTERED AND SET DEFAULT VALUE IF NOT 	*/
/* IDVAR NEEDS A VALUE BEFORE %varexst CALL 		*/
/********************************************************/
%if %length(&IDVAR)= 0 %then %do;
       %let IDVAR  = usubjid;
%end; 

/****************************************/
/* CHECK THAT VARIABLES EXIST 		*/
/* ADD varexst FOR usubjid IN EVTDATA 	*/
/*********************************************/

%varexst(DATA	= &EVTDATA., 
	 VARS	= &IDVAR &COLVAR &GRPVAR &EVTVAR, 
	 ADDMSG	= %str(for FLAGSUM.  Please include this variable in &EVTDATA and rerun the program.),
	 IFERR	= endsas);

%varexst(DATA	= &COLDATA., 
	 VARS	= &IDVAR &COLVAR, 	
	 ADDMSG	= %str(when both are specified.  Please include this variable in &COLDATA and rerun the program.),	
	 IFERR	= endsas);

/***********************************************/
/* CHECK IF VARS EXIST IF LISTED IN PARAMETERS */
/***********************************************/
%if %length(&SUBVAR) > 0 or 
    %length(&ALLFLG) > 0 or 
    %length(&GRPFLG) > 0 or
    %length(&EVTFLG) > 0
%then %do;
      %local VAREXSTLIST;
      %let   VAREXSTLIST=;

      %if %length(&SUBVAR) > 0 %then %do;
	 %let VAREXSTLIST= &SUBVAR;
      %end;
      %if %length(&ALLFLG) > 0 %then %do;
         %let VAREXSTLIST= &VAREXSTLIST &ALLFLG;
      %end;
      %if %length(&GRPFLG) > 0 %then %do;
         %let VAREXSTLIST= &VAREXSTLIST &GRPFLG;
      %end;
      %if %length(&EVTFLG) > 0 %then %do;
         %let VAREXSTLIST= &VAREXSTLIST &EVTFLG;
      %end;

      %varexst(DATA	= &EVTDATA., 
	       VARS	= &VAREXSTLIST.,   
	       ADDMSG	= %str(when specified.  Please include &VAREXSTLIST in &EVTDATA and rerun the program.), 		
	       IFERR	= endsas);
%end;

/************************************************************/
/* ADD COLVAR CHECK TO SEE IF IT MIGHT BE A GRADE VARIABLE  */
/* IF SO, REMIND USER IT SHOULD BE LISTED IN SUBVAR INSTEAD */
/************************************************************/
%if %index(%lowcase(&COLVAR), grad) or 
    %index(%lowcase(&COLVAR), grd)  %then %do; 
  	%put %upcase(warning:) The given COLVAR seems to represent the grade of your EVTVAR.; 
	%put %upcase(warning:) This may be a mistake.  Typically, COLVAR lists a treatment or arm type variable.;
	%put %upcase(warning:) Please use the following UNIX command to see examples:  read2me -w eventsum ;
	%put %upcase(warning:) They will be at the end of the WORD document and show tables expected from flagrpt.;
%end; 

/********************************************************/
/* ADD DEBUG OPTION TO PARAMETERS 			*/
/********************************************************/
%if %length(&DEBUG)= 0 %then %let DEBUG = Y;
%else 			     %let DEBUG = %substr(%upcase(&DEBUG),1,1);

%if &DEBUG = Y %then %do;
   options mprint mlogic mlogicnest source2 symbolgen; 
%end;

/********************************************************/
/* SET DEFAULT FOR OUTDATA IF BLANK                     */
/* SET DEFAULT FOR LSTMORE IF BLANK			*/
/* SET DEFAULT FOR PCTFMT  IF BLANK                     */
/*	     CHECK PCTFMT  EXISTS			*/
/* SET DEFAULT FOR PCTMODE IF BLANK                     */
/* 	    ADJUST PCTMODE AS NEEDED			*/
/********************************************************/
%if %length(&OUTDATA) = 0 %then %let OUTDATA = flagend;

%if %length(&LSTMORE)= 0 %then  %let LSTMORE = Y;
%else                           %let LSTMORE = %substr(%upcase(&LSTMORE),1,1);

/* FYI - CANNOT CHECK EXISTENCE OF FORMATS LIKE 4. OR 7.2 */
/* 	 AND THESE ARE EXPECTED TO BE USED IN PCTFMT	  */
/* 	 SO THERE IS NO CHECK FOR EXISTENCE OF FMT  	  */
/**********************************************************/
%if %length(&PCTFMT) = 0 %then %do;
   proc format;
        value flgsmft
             0<-<.5 = "<0.5"
             other  = [4.]
             ;
   run;
   %let PCTFMT = flgsmft4.;
%end;
%else %do;
   %if %sysfunc(find(&PCTFMT,%str(.))) = 0 %then %do;
      %let PCTFMT = %sysfunc(strip(&PCTFMT%str(.)));
   %end;
   %put &PCTFMT;
%end;

/* DELETE PUNCTUATION AND DIGITS FROM PCTFMT VALUE  */
/* THIS GIVES NAME OF FORMAT TO SEARCH IF IT EXISTS */
%let CHKFMT = %sysfunc(compress(&PCTFMT,,dp));

/* CHECK FORMAT IF NOT SOLELY A NUMERIC FORMAT, LIKE 7. or 7.2 */
%if %length(&CHKFMT) ^= 0 %then %do;
   proc sql noprint;
      create table chkflagfmt
      as select libname, fmtname, source, fmttype
      from dictionary.formats
      where fmtname="%upcase(&CHKFMT)"
      ;
  quit;

  %if %nobs(chkflagfmt) = 0 %then %do;
     %put %upcase(error:) The following format does not exist:  &PCTFMT.; 
     %put %upcase(error:) Please enter a format that exists in  PCTFMT.; 
     %put %upcase(error:) Once corrections are made, please try running FLAGSUM again.;
     %put %upcase(error:) FLAGSUM will now terminate.;
     %abort abend 6;
  %end;
%end;

%if %length(&PCTMODE) = 0 %then %let PCTMODE = 1;
%if &PCTMODE^=0 and &PCTMODE^=1 and
    &PCTMODE^=2 and &PCTMODE^=3 and
    &PCTMODE^=4 and &PCTMODE^=5 and
    &PCTMODE^=6 and &PCTMODE^=7 and
    &PCTMODE^=8 and &PCTMODE^=9 and
    &PCTMODE^=10
                          %then %let PCTMODE = 1;

/********************************************************/
/* CHANGE PCTMODE CODES ACCORDINGLY 			*/
/* VALUES OF 6, 7, and 10 CHANGE IF THERE IS NO SUBVAR 	*/
/********************************************************/
%if %length(&SUBVAR) = 0 %then %do;
   %if &PCTMODE = 6  %then %let PCTMODE = 1;;
   %if &PCTMODE = 7 or 
       &PCTMODE = 10 %then %let PCTMODE = 3;;
%end;

/*******************************************************************************/
/* EVARTYPE = C for char or N for numeric                                      */
/* EVARLEN = LENGTH OF VARIABLE TO CREATE "All Events" FMT WITH CORRECT LENGTH */
/* GET CHARACTERISTICS OF eventvar, grpvar, colvar, subvar, AND idvar          */
/*******************************************************************************/
/* OPEN DATASETS */
/*****************/
   %let DSID1    = %sysfunc(open(&EVTDATA));
   %let DSID2	 = %sysfunc(open(&COLDATA));

/********************************************************************************/
/* RETRIEVE CHARACTERISTICS OF VARIABLES NEEDED FOR CODING BELOW   		*/
/* IDVAR NEEDS THIS TO AVOID "Multiple length" WARNING THAT FAIL VALIDLOG 	*/
/* LEAVE VARLEN COMMENTED FOR NOW IN CASE NEED IT LATER                         */
/********************************************************************************/

/*****************************************************************************/
/* SET COLVAR RELATED ITEMS - VARTYPE, VARLEN, VARFMT                        */
/* CHECK THAT EVTDATA COLVAR INFO MATCHES COLDATA COLVAR INFO */
/*****************************************************************************/
   %let COLVTYPE1 = %sysfunc(vartype(&DSID1,%sysfunc(varnum(&DSID1,&COLVAR))));
   %let COLFMT1   = %sysfunc(varfmt( &DSID1,%sysfunc(varnum(&DSID1,&COLVAR))));

   %let COLVTYPE2 = %sysfunc(vartype(&DSID2,%sysfunc(varnum(&DSID2,&COLVAR))));
   %let COLFMT2   = %sysfunc(varfmt( &DSID2,%sysfunc(varnum(&DSID2,&COLVAR))));

/* CHECK COLVAR VALUES IN EVTDATA AND COLDATA MATCH */
%if %sysfunc(strip(&COLVTYPE1)) ^= %sysfunc(strip(&COLVTYPE2)) %then %do;
   %put %upcase(error:) The variable type for &COLVAR in &EVTDATA and &COLDATA do not match.;
   %put %upcase(error:) EVTDATA = &COLVTYPE1 and COLDATA = &COLVTYPE2;
   %put %upcase(error:) ;
   %put %upcase(error:) Please make sure &COLVAR in &COLDATA and &EVTDATA are the same variable.;
   %put %upcase(error:) FLAGSUM will now terminate.;
   %abort abend 7;
%end;

/* CHECK FORMATS FOR COLVAR ARE THE SAME IN EVTDATA AND COLDATA           */
/*   FMTEQ = 1 INDICATES THE MACRO NEEDS TO END BECAUSE COLFMTS NOT EQUAL */
%let FMTEQ = 0;

%if %length(&COLFMT1) > 0 and %length(&COLFMT2) > 0 %then %do;
   %if %sysfunc(strip(&COLFMT1)) ^= %sysfunc(strip(&COLFMT2)) %then %do;
       %let FMTEQ = 1;
   %end;
%end;
%else %if %length(&COLFMT1) > 0 or  %length(&COLFMT2) > 0 %then %do;

/* AT THIS POINT, IF ONE HAS A VALUE THEN THE OTHER DOES NOT */
/* SO THEY ARE NOT EQUAL */
       %let FMTEQ = 1;
%end;

/* PRINT MESSAGE AND TERMINATE IF THERE IS A PROBLEM */
%if &FMTEQ = 1 %then %do;
       %put %upcase(error:) The formats for &COLVAR in &EVTDATA and &COLDATA do not match.;
       %put %upcase(error:) EVTDATA = &COLFMT1;;
       %put %upcase(error:) COLDATA = &COLFMT2;;
       %put %upcase(error:) ;
       %put %upcase(error:) Please make sure &COLVAR in &COLDATA and &EVTDATA are the same variable.;
       %put %upcase(error:) FLAGSUM will now terminate.;
       %abort abend 8;
%end;

/************************************************************************/
/* "TOTAL" BECOMES THE VALUE OF THE TOTAL COLUMNS IN FREQ DATASETS 	*/
/************************************************************************/
   %let COLTOTVAL = TOTAL;

/****************************/
/* SET SUBVAR RELATED ITEMS */
/****************************/
%if %length(&SUBVAR) > 0 %then %do;

/************************************************************************/ 
/* "All" BECOMES THE VALUE OF THE TOTAL COLUMNS IN FREQ DATASETS      	*/
/************************************************************************/
   %let SUBTOTVAL = All;
%end;

/***********************************************/
/* SET IDVAR RELATED ITEMS - VARTYPE, VARLEN  */
/***********************************************/
/******************************************************************/
/* SET LENGTH TO LONGEST VALUE TO AVOID "Multiple length" WARNING */
/* CHECK THAT IDVAR IS THE SAME TYPE IN EVTDATA AND COLDATA       */
/******************************************************************/

   %let IDEVTYPE  = %sysfunc(vartype(&DSID1, %sysfunc(varnum(&DSID1,&IDVAR))));
   %let IDEVLEN   = %sysfunc(varlen( &DSID1, %sysfunc(varnum(&DSID1,&IDVAR))));

   %let IDCVTYPE  = %sysfunc(vartype(&DSID2,%sysfunc(varnum(&DSID2,&IDVAR))));
   %let IDCVLEN   = %sysfunc(varlen( &DSID2,%sysfunc(varnum(&DSID2,&IDVAR))));

%if %sysfunc(strip(&IDEVTYPE)) ^= %sysfunc(strip(&IDCVTYPE)) %then %do;
   %put %upcase(error:) The &IDVAR variable type in EVTDATA does not match that in COLDATA.;
   %put %upcase(error:) The &IDVAR variable type in EVTDATA is &IDEVTYPE..;
   %put %upcase(error:) The &IDVAR variable type in COLDATA is &IDCVTYPE..;
   %put %upcase(error:) C is character.  N is numeric.;
   %put %upcase(error:) Please correct the variable types and try again.;
   %abort abend 9;
%end;

/* GET MAX LENGTH OF IDVAR REGARDLESS IF VALUES DIFFERENT */
%let IDLEN = %sysfunc(max(&IDEVLEN,&IDCVLEN));

%if &IDEVTYPE = C %then %do;
    %let FLGIDLEN = length &IDVAR $&IDLEN..;
%end;
%else %do;
    %let FLGIDLEN = length &IDVAR &IDLEN..;
%end;

/******************/
/* CLOSE DATASETS */
/******************/
   %let RC       = %sysfunc(close(&DSID1));
   %let RC2	 = %sysfunc(close(&DSID2));

/*************************************************************************************************
   COLDATA - NEEDS RECS TO CALCULATE N= TOTALS AKA THE DENOMINATOR FOR PCTS
   EVTDATA - ONLY AE/OCCURRENCE DATA AND SUBVAR (LIKELY GRADE) VALUES

   IF COLVAR OR SUBVAR ARE NUMERIC, ANOTHER VARIABLE NEEDS TO BE MADE THAT IS THE 
	ACTUAL VALUE WITHOUT THE FORMAT TO USE THE NOBS MACRO CORRECTLY.
**************************************************************************************************/

/*******************************************/
/* CHECK FOR DUPLICATE RECORDS IN &COLDATA */
/*******************************************/
%dupkey( DATA   = &COLDATA
        ,BYVARS = &IDVAR &COLVAR
        ,IFDUP  = ENDSAS
        );

/**********************************************/
/* REMOVE RECS WITH MISSING VALUES IN &COLVAR */
/**********************************************/
data misscol;
   set &COLDATA
       %if %length(&COLWHERE) > 0 %then %do;
	  (where=(&COLWHERE))
       %end;
       ;
   if   missing(&COLVAR) or 
	missing(&IDVAR)  then output; 
run;

/*****************************************************/
/* IF RECS IN misscol THEN PROGRAM STILL RUNS TO END */ 
/*****************************************************/
%let MISSNUM = %nobs(misscol);
%if &MISSNUM > 0 %then %do;
   %put %upcase(note: (cbar)) -- There were records with missing &COLVAR or &IDVAR values	 ;
%if %length(&COLWHERE) > 0 %then %do;
   %put %upcase(note: (cbar)) -- when COLWHERE is:  &COLWHERE..				 	 ;
%end;
   %put %upcase(note: (cbar)) -- These were omitted from the counts and denominator calculations.;
   %put %upcase(note: (cbar)) -- Please check the .lst for teh omitted values.			 ;
   %put %upcase(note: (cbar)) -- FLAGSUM will continue....					 ; 
   %put %upcase(note: (cbar)) -- ++++ &COLDATA HAD RECORDS WITH MISSING VALUES ++++++++++++++++++;

   title "RECS WITH MISSING &COLVAR OR &IDVAR VALUES FROM &COLDATA";
   proc print data= misscol;
   run;
%end;

/* CHECK IDVAR/COLVAR COMBINATIONS IN EVTDATA AND COLDATA MATCH */
proc sort data = &COLDATA 
    	  %if   %length(&COLWHERE) > 0 %then %do;
	      (where = (&COLWHERE))
    	  %end;
	  out  = colcolv (keep = &IDVAR &COLVAR)
	  nodupkey;
     by &IDVAR &COLVAR;
run;

proc sort data = &EVTDATA 
          %if   %length(&COLWHERE) > 0 %then %do;
              (where = (&COLWHERE))
          %end;
	  out  = evtcolv ( keep = &IDVAR &COLVAR)
	  nodupkey;
     by &IDVAR &COLVAR;
run;

/* WILL NEED A SPECIFIC EXAMPLE TO TEST THIS! */
data colprobs;
   &FLGIDLEN;
   merge colcolv (in = incolv)
	 evtcolv (in = inevtv);
   by &IDVAR &COLVAR;
   length in_col in_evt 3.;
   if incolv and inevtv then delete;
   if incolv then in_col = 1;
   if inevtv then in_evt = 1;
run;

/* ONLY PRINT OUT IF THERE IS A RECORD THAT IS ONLY IN EVTDATA, aka in_evt = 1 */
%let BADCOLN = %nobs(colprobs,%str(in_evt = 1));
%if &BADCOLN > 0 %then %do;

   %put %upcase(error:) Please look at the end of the .lst for those records 		 ;
   %put %upcase(error:) in &EVTDATA that did not match one in &COLDATA.                  ;
   %put %upcase(error:) ==============================================================   ;

%if %length(&COLWHERE) > 0 %then %do;
   %put %upcase(error:)                                                                  ;
   %put %upcase(error:) If known problem records are not seen, it may be because COLWHERE;
   %put %upcase(error:) was set to:  &COLWHERE..					 ;
%end;

   %put %upcase(error:) -- Please check the datasets and correct the unmatching values before running again.		;
   %put %upcase(error:) -- FLAGSUM will now terminate.									;

   title  "WE HAVE ONE OF THREE PROBLEMS:"								;
   title2 "1. There are recs in &EVTDATA with a value for &IDVAR that does match a record in &COLDATA."	; 
   title3 "2. There are recs in &EVTDATA with a value for &COLVAR that does match a record in &COLDATA.";
   title4 "3. Both of the 2 items above contained in one record of &EVTDATA."				;
   title5 "-------------------------------------------------------------------------------------------" ;
   title6 "IF in_col= 1 THEN THE RECORD COMES FROM &COLDATA"						;
   title7 "IF in_evt= 1 THEN THE RECORD COMES FROM &EVTDATA"						;
   title8 "THIS INFORMATION SHOULD HELP DETERMINE THE ISSUE."						;

   proc print data= colprobs;
   run;
   %abort abend 10;

%end;

/* PRINT OUT MISSING IDVAR VALUES */
/* EVTDATA MISSING RECS FOR:  IDVAR, COLVAR, GRPVAR, EVTVAR, SUBVAR */
data evtidmiss;
   set &EVTDATA
       %if %length(&COLWHERE) > 0 %then %do;
 	   (where=(&COLWHERE))
       %end;
       ;
   if missing(&IDVAR)  or missing(&COLVAR) or
      missing(&GRPVAR) or missing(&EVTVAR)
      %if %length(&SUBVAR) > 0 %then %do;
  	 or missing(&SUBVAR)
      %end;
      then output;
run;

%let EVTIDMISSN = %nobs(evtidmiss);
%if &EVTIDMISSN > 0 %then %do;
   %put %upcase(note: (cbar)) -- In &EVTDATA, there are missing values for the following variables:  	;
   %put %upcase(note: (cbar)) -- &IDVAR. &COLVAR. &GRPVAR. &EVTVAR. &SUBVAR.				;	

%if %length(&COLWHERE) > 0 %then %do;
   %put %upcase(note: (cbar)) -- When COLWHERE is:  &COLWHERE..						;
%end;

   %put %upcase(note: (cbar)) -- These records will not be counted at all.				;
   %put %upcase(note: (cbar)) -- Please check the data and ensure these are fixed in the future.	;
   %put %upcase(note: (cbar)) -- FLAGSUM will continue.....						;
   %put %upcase(note: (cbar)) -- ====== &EVTDATA HAD RECORDS WITH MISSING VALUES =======================;

   title  "RECS WITH MISSING VALUES IN &EVTDATA FOR THE FOLLOWING VARIABLES:  &IDVAR &COLVAR &GRPVAR &EVTVAR &SUBVAR";
%if %length(&COLWHERE) > 0 %then %do;
   title2 "WHEN COLWHERE IS:  &COLWHERE";
%end;
   proc print data= evtidmiss;
      var &IDVAR. &COLVAR. &GRPVAR. &EVTVAR. &SUBVAR. ;
   run;

%end;

/*********************************************/
/* GET LIST OF FORMATTED VALUES OF COLVAR    */
/* CALCULATE DENOM FOR TOTAL COLVAR COLUMN   */
/* ONLY WHERE COLVAR IS NON-MISSING	     */
/*********************************************/
/* THERE CANNOT BE DUPLICATES IN COLDATA AT  */
/* THIS POINT BECAUSE %dupkey WILL endsas    */
/* IF ANY FOUND  			     */
/* *******************************************/
/* AT THIS POINT, WE KNOW     		     */
/* COLVAR VALUES IN COLDATA MATCH THOSE IN   */
/* IN EVTDATA, SO COLDATA COUNTS ARE         */
/* ACCURATE TO COLVAR VALUES IN EVTDATA      */
/*********************************************/
/* PROC SQL HAS feedback OPTION THAT SHOWS MORE ABOUT WHAT IT IS DOING */
/***********************************************************************/
proc sql noprint; * feedback;
   select distinct &COLVAR into : CLIST separated by '@' 
   from &COLDATA
   where ^(missing(&COLVAR)) and ^(missing(&IDVAR)) 
      %if %length(&COLWHERE) > 0 %then %do;
  	 and &COLWHERE
      %end;
   ;
quit;

/* GET NUMBER OF ITEMS IN CLIST */
   %let CLISTMAX = %sysfunc(countw(%nrbquote(&CLIST),@));

/* CREATE VALID VARNAME LIST IN A MACRO VAR FOR COLVAR VALUES 		*/
/* ALONG WITH OTHER MACRO VAR FOR MACRO VARS WITH DENOM VALUES 		*/ 
/* NEED TOP LENGTHS FOR VVALUE AND VARNAME VALUES FOR length STATEMENT 	*/
/* IN A DATA STEP 							*/
/************************************************************************/
   %let   DENOMLIST=;

/*******************************************************************/
/* DEFAULT VALUE PICKED BY CONSIDERING A NUMBER WITH 3 PLACES 	   */
/* ADDING AN _ TO THE BEGINNING OF THE VAR NAME AND _10 TO THE END */
/*******************************************************************/
   %let   CVNMTOP  = 7;

/* PROCESS COLVAR VALUES */
%do A=1 %to &CLISTMAX;
        %local  THISCVAL&A ;
        %let    THISCVAL&A = %qscan(%nrbquote(&CLIST), &A, @);

/*******************************************************************************/
/* REMOVE SPACES AND PUNCTUATION FROM VALUE SO IT CAN BE A VALID VARIABLE NAME */ 
/*******************************************************************************/
	%local  VARCNAME&A ; 
	%let    VARCNAME&A = %sysfunc(compress("&&THISCVAL&A", ,sp))_&A;

/********************************************************************************/
/* CHECK THAT VARCNAME VALUE IS NOT TOO LARGE: OVER 27 CHARS IN LENGTH 		*/
/* SO THAT IT CAN ACCOMMODATE ADDING "DENOM" (5 CHARS LONG) TO THE NAME 	*/
/* SAS MAX VARNAME LENGTH = 32 							*/
/* IF CLISTMAX >= 10 (ASSUME IT WILL NOT BE 100 OR MORE) THEN 			*/
/* LENGTH OF COLVAR VALUE MUST BE <= 24 BECAUSE ADDING "_10" ON END OF NAME 	*/
/* ELSE IT IS <= 25 (ADDING "_9" ON END OF NAME) 				*/
/********************************************************************************/
	%if %length(&&VARCNAME&A) > 27 %then %do;
	   %if &CLISTMAX >= 10 %then %do;
		%put %upcase(error:) The following &COLVAR value is over 24 characters long.		;
		%put %upcase(error:)   %sysfunc(compress("&&THISCVAL&A", ,sp)) 				; 
		%put %upcase(error:)   ------------------------						;
		%put %upcase(error:) -- LINE ABOVE IS 24 CHARS FOR COMPARISON				;
		%put %upcase(error:) FLAGSUM only caught the first instance in &COLVAR.			; 
		%put %upcase(error:) If there are others, please change them before re-running FLAGSUM.	;
	   %end;
	   %else %do;
                %put %upcase(error:) The following &COLVAR value is over 25 characters long.		;
                %put %upcase(error:)   %sysfunc(compress("&&THISCVAL&A", ,sp)) 				; 
                %put %upcase(error:)   -------------------------					;
                %put %upcase(error:) -- LINE ABOVE IS 25 CHARS FOR COMPARISON 				; 
                %put %upcase(error:) FLAGSUM only caught the first instance in &COLVAR.			;
                %put %upcase(error:) If there are others, please change them before re-running FLAGSUM.	;
	   %end;
	   %abort abend 11;;
	%end;

/* IF FIRST CHARACTER IS A NUMBER ADD AN UNDERSCORE AS FIRST CHAR */
  	%if %sysfunc(anydigit(%sysfunc(substr(&&VARCNAME&A,1,1)))) > 0 %then %do;
	   %let VARCNAME&A = _&&VARCNAME&A;
	%end;

/* GET COUNT  OF RECS WITH COLVAR VALUE */
/* GET LENGTH OF LONGEST   COLVAR VALUE */

   data _null_;
       set &COLDATA (where=( ^(missing(&IDVAR))
	   %if %length(&COLWHERE) > 0 %then %do;
 		and &COLWHERE
	   %end;
		    ))
		end = last;

       retain nobs_&A 0;

       if strip(vvalue(&COLVAR)) = "%nrbquote(&&THISCVAL&A)" then
	   nobs_&A+1;
       if last then do;
	   call symput("DENOM&&VARCNAME&A",nobs_&A);
       end;
   run;

/* MAKE MACRO VARS TO WRITE CODE FOR flagdenom DATASET */
   %local FLAGDENOM&A LBLDENOM&A;

   %let   FLAGDENOM&A = %str(denom&&VARCNAME&A = &&&&DENOM&&VARCNAME&A);
   %let   LBLDENOM&A  = %str(denom&&VARCNAME&A = "DENOM FOR:  &&THISCVAL&A");

   %let   DENOMLIST   = &DENOMLIST denom&&VARCNAME&A;

/* NEED A SIMILAR COMMA DELIMITED LIST FOR CALCULATING LENGTH OF SUM OF ALL VALUES */
/* VALUE WILL BE USED IN A sum() FUNCTION IN MACRO CODE */
   %if &A = 1 %then %do;
        %local MACCOMLIST;
        %let   MACCOMLIST =  &&&&DENOM&&VARCNAME&A;
   %end;
   %else %do;
        %let   MACCOMLIST = &MACCOMLIST,&&&&DENOM&&VARCNAME&A;
   %end;

   %put &MACCOMLIST;

   %if %length(&&VARCNAME&A) > &CVNMTOP %then %do;   
      %let CVNMTOP    = %eval(%length(&&VARCNAME&A)+4); 
   %end; 

/* CHECK IT OUT */
   %put CLIST: 	     &CLISTMAX %superq(CLIST) &&VARCNAME&A ;
   %put FLAGDENOM&A: &&FLAGDENOM&A;
   %put LBLDENOM&A:  &&LBLDENOM&A;
%end;

/* GET TOTAL FOR denomTOTAL IN FINAL DATASET AND LENGTH OF THIS TOTAL */
%local MACCOMTOT TOTLEN;
%let MACCOMTOT = %sysfunc(sum(&MACCOMLIST));
%let TOTLEN    = %length(&MACCOMTOT);

%put MACCOMTOT: &MACCOMTOT;
%put TOTLEN:    &TOTLEN;


/* RETRIEVE FORMAT WIDTHS FOR VARIABLES IN &EVTDATA */
   data _null_;
       set &EVTDATA (  
                      where=( ^(missing(&COLVAR)) and ^(missing(&IDVAR))
                      %if %length(&COLWHERE) > 0 %then %do;
                              and &COLWHERE
                      %end;
		            )
		    )
                    ;
/* GET FORMAT LENGTHS TO CREATE REPORTING (rpt: AND grpev1col) VARIABLES */
       call symput("GRPVARFTW",strip(vformatw(&GRPVAR)));			
       call symput("EVTVARFTW",strip(vformatw(&EVTVAR)));
       call symput("COLVARFTW",strip(vformatw(&COLVAR)));
%if %length(&SUBVAR) > 0 %then %do;
       call symput("SUBVARFTW",strip(vformatw(&SUBVAR)));
%end;
   run;

/* CREATE LENGTH OF grpev1col VARIABLE HERE, NAMED GRPEVFTW */
%let GRPEVFTW = %eval(%sysfunc(max(&GRPVARFTW,&EVTVARFTW))+27);

/*****************************************************************/
/* CHECK TO DETERMINE IF ANY EVENT HAS MULTIPLE GROUPS		 */
/*****************************************************************/
/* SORT BY EVENT FIRST SO IF IT IS IN MULTIPLE GROUPS 	*/
/* THE COUNT WILL BE > 1 BELOW 				*/
/********************************************************/
proc sort data = &EVTDATA (keep= &EVTVAR &GRPVAR ) 
	  out  = manygrps 
	  nodupkey;
    by &EVTVAR &GRPVAR;
run;

%if &DEBUG = Y %then %do;
   title "ANY EVENTS WITH MULTIPLE GROUP VALUES";
   proc print data= manygrps;
   run;
%end;

data manycount (where=( count > 1));
    set manygrps;
    by &EVTVAR &GRPVAR;
    if first.&GRPVAR;
    if first.&EVTVAR then 
	 count= 1;
    else count+1;
run;

%let MANYGRPS = %nobs(manycount);

%if &MANYGRPS > 0 %then %do; 

   %put %upcase(note: (cbar)) -- The &EVTVAR variable has multiple possible &GRPVAR values.        ;
   %put %upcase(note: (cbar)) -- Please check the .lst file to see which &EVTVAR values            ;
   %put %upcase(note: (cbar)) -- have multiple &GRPVAR values and correct any unexpected results.  ;
   %put %upcase(note: (cbar)) =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-                   ;
   %put %upcase(note: (cbar)) -- FLAGSUM will continue...                                          ;
   %put %upcase(note: (cbar)) >>>> &EVTVAR HAS AT LEAST ONE VALUE WITH MULTIPLE &GRPVAR VALUES <<<<;

   data manyprint;
      merge manycount (in= incount)
	    manygrps  (in= ingrps)
      ;
      by &EVTVAR; 
      if incount and ingrps;
   run;

   title "EVENTS WITH MORE THAN ONE GROUP";
   proc print data= manyprint;
   run; 

%end; 

/************************/
/* GET EVENT TOTALS 	*/
/************************/
/************************************************************************************************/   
/*  SIMPLE PROC FREQ TO GET ALL OF THE TOTALS THAT ARE NEEDED ; 				*/
/*  LATER THESE WILL BE MERGED TOGETHER AND THE PERCENTAGES OBTAINED BY A SIMPLE 		*/
/*       QUOTIENT WITH NUMBERS OF RANDOMIZED SUBJECTS ; 					*/
/*  NOTE: NEED TO GET SOC AND OVERALL TOTALS SEPARATELY AS THE HIGHEST GRADE 			*/
/*        BY PT/EVENT FOR AN INDIVIDUAL MAY NOT BE THE HIGHEST GRADE PT/EVENT BY SOC ; 		*/
/************************************************************************************************/
/* FYI - IF THERE ARE NUMBERS COMMENTED OUT, USUALLY THESE REFER TO TOTALS vs SPECIFIC VALUES 	*/
/* MAY NEED THIS INFO LATER...BELOW IS WHAT I BELIEVE I UNDERSTAND: 				*/
/* 1111 AND 111 ARE "EVENT" FREQS, 								*/
/* 1110 AND 110 ARE "TOTAL" FREQS - ANY NUMBER ENDING IN 0 IS A TOTAL 				*/
/************************************************************************************************/    
   ods listing exclude all;

/* SET MACRO VARIABLE VALUES IN CASE THEY ARE NOT CREATED LATER */
%let SETALL =;
%let SETGRP =;
%let SETEVT =;

/* GET COUNTS WHERE IDVAR, COLVAR, GRPVAR, EVTVAR, and SUBVAR ARE NOT MISSING */
** TOTALS FOR THE OVERALL ROW/ALL;
%if %length(&ALLFLG) > 0 %then %do;
   ods output CrossTabFreqs = ALL_N (where= (frequency > 0));
   proc freq data= &EVTDATA         (where= (&ALLFLG   = "Y"
				     and ^(missing(&IDVAR  ))
				     and ^(missing(&COLVAR ))
				     and ^(missing(&GRPVAR ))
                                     and ^(missing(&EVTVAR ))
   				%if %length(&SUBVAR) > 0 %then %do;
				     and ^(missing(&SUBVAR))
				%end;
					   )
				    );
%if %length(&COLWHERE) > 0 %then %do;
      where &COLWHERE;
%end;

%if %length(&SUBVAR) > 0 %then %do;
      table &COLVAR*&SUBVAR 	/ nopercent norow nocol missprint;
   run;
%end;
%else %do;
      table &GRPVAR*&COLVAR 	/ nopercent norow nocol missprint;
   run;

/********************************************************/
/* NEED TO NOT HAVE DOUBLE VALUES FOR "Overall" ROW 	*/ 
/* SINCE ALL_N AND GRP_N ARE GRPVAR*COLVAR 		*/
/* WHERE missing(GRPVAR) ARE VALUES THAT OVERLAP AND 	*/
/* ONLY WANT VALUES FROM ALL_N FOR "Overall" ROW 	*/
/********************************************************/
   data ALL_N;
     set ALL_N;
     if missing(&GRPVAR);
   run;

%end; /* ELSE DO FOR %length(&SUBVAR) > 0 */
%end; /* IF %length(&ALLFLG) > 0	  */

** TOTALS FOR OVERALL GROUP/SOC ROWS;
%if %length(&GRPFLG) > 0 %then %do;
   ods output CrossTabFreqs = GRP_N (where= (frequency > 0));
   proc freq data= &EVTDATA         (where= (&GRPFLG   = "Y"
                                     and ^(missing(&IDVAR  ))
                                     and ^(missing(&COLVAR ))
                                     and ^(missing(&GRPVAR ))
                                     and ^(missing(&EVTVAR ))
                                %if %length(&SUBVAR) > 0 %then %do;
                                     and ^(missing(&SUBVAR))
                                %end;
					    )
				    );

%if %length(&COLWHERE) > 0 %then %do;
      where &COLWHERE;
%end;

%if %length(&SUBVAR) > 0 %then %do;
      table &GRPVAR*&SUBVAR*&COLVAR / nopercent norow nocol missprint;
   run;
%end;
%else %do;
      table &GRPVAR*&COLVAR 	    / nopercent norow nocol missprint;
   run;

/* CANNOT HAVE DOUBLE VALUES FOR "Overall" ROW 	        */ 
/* SINCE ALL_N AND GRP_N ARE GRPVAR*COLVAR 		*/
/* WHERE missing(GRPVAR) ARE VALUES THAT OVERLAP AND 	*/
/* ONLY WANT VALUES FROM ALL_N FOR "Overall" ROW 	*/
/********************************************************/
   data GRP_N;
     set GRP_N;
     if ^missing(&GRPVAR); 
   run;

%end; /* ELSE DO FOR %length(&SUBVAR) > 0  */
%end; /* IF %length(&GRPFLG) > 0 	   */

/* TOTALS FOR EVENT ROWS */
%if %length(&EVTFLG) > 0 %then %do;
   ods output CrossTabFreqs = EVT_N (where= (frequency > 0) drop= Table);
   proc freq data= &EVTDATA         (where= (&EVTFLG    = "Y"
                                     and ^(missing(&IDVAR  ))
                                     and ^(missing(&COLVAR ))
                                     and ^(missing(&GRPVAR ))
                                     and ^(missing(&EVTVAR ))
                                %if %length(&SUBVAR) > 0 %then %do;
                                     and ^(missing(&SUBVAR))
                                %end;
					    )
				    );

%if %length(&COLWHERE) > 0 %then %do;
      where &COLWHERE;
%end;

%if %length(&SUBVAR) > 0 %then %do;
      table &GRPVAR*&EVTVAR*&SUBVAR*&COLVAR / nopercent norow nocol missprint;
   run;
%end;
%else %do;
      table &GRPVAR*&EVTVAR*&COLVAR 	     / nopercent norow nocol missprint;
   run;

/********************************************************/
/* NEED TO NOT HAVE DOUBLE VALUES FOR GROUP ROWS     	*/
/* WHERE missing(EVTVAR) ARE VALUES THAT OVERLAP WITH   */
/* THOSE IN GRP_N AND ONLY WANT VALUES FROM GRP_N FOR 	*/
/* GROUP ROWS						*/
/********************************************************/
   data EVT_N;
     set EVT_N;
     if ^missing(&EVTVAR);
   run;

%end;  /* ELSE DO FOR %length(&SUBVAR) > 0 */
%end;  /* IF %length(&EVTFLG) > 0 	   */

   ods listing select all;

/********************************************************/
/* PROC PRINT ANY DATASETS OF FREQUENCIES OBTAINED	*/
/* THIS NEEDED TO BE SEPARATE FROM CODE ABOVE BECAUSE 	*/
/* OF THE ods listing select all STATEMENT		*/
/* ONLY PRINT IF LSTMORE = Y				*/
/********************************************************/
%if %length(&ALLFLG) > 0 %then %do;

   %if &LSTMORE = Y %then %do;
      title  "PRINT OF FREQS FOR THE ALL_N/OVERALL ROW";
      title2 "MISSING VALUES IN &COLVAR ARE FOR TOTAL COLUMN";
    %if %length(&SUBVAR) > 0 %then %do;
      title3 "MISSING VALUES IN &SUBVAR ARE TOTAL COLUMNS FOR EACH &COLVAR VALUE";
    %end;
      proc print data= All_N; 
      run;  

      title "CONTENTS FOR ALL_N/OVERALL ROW";
      proc contents data= All_N;
      run;
   %end;

/* CODE TO USE IN set STATEMENT BELOW THAT PUTS ALL FREQS TOGETHER */
   %let SETALL = %str(ALL_N (keep= &COLVAR &SUBVAR frequency));

%end;

%if %length(&GRPFLG) > 0 %then %do;

   %if &LSTMORE = Y %then %do;
      title  "PRINT OF FREQS FOR SOC_N/GROUP ROWS";
      title2 "MISSING VALUES IN &COLVAR ARE FOR TOTAL COLUMN";
    %if %length(&SUBVAR) > 0 %then %do;
      title3 "MISSING VALUES IN &SUBVAR ARE TOTAL COLUMNS FOR EACH &COLVAR";
    %end;

      proc print data= GRP_N; *(obs=20);
      run; 

      title "CONTENTS FOR SOC_N/GROUP ROWS";
      proc contents data= GRP_N;
      run;
   %end;

/* CODE TO USE IN set STATEMENT BELOW THAT PUTS ALL FREQS TOGETHER */
   %let SETGRP = %str(GRP_N (keep= &COLVAR &SUBVAR frequency &GRPVAR));

%end;

%if %length(&EVTFLG) > 0 %then %do;

   %if &LSTMORE = Y %then %do;
      title  "PRINT OF FREQS FOR PT_N/EVENT ROWS";
      title2 "MISSING VALUES IN &COLVAR ARE FOR TOTAL COLUMN";
    %if %length(&SUBVAR) > 0 %then %do;
      title3 "MISSING VALUES IN &SUBVAR ARE TOTAL COLUMNS FOR EACH &COLVAR";
    %end;

      proc print data= EVT_N; *(obs=20);
      run;

      title "CONTENTS FOR PT_N/EVENT ROWS";
      proc contents data= EVT_N;
      run;
   %end;

/* CODE TO USE IN set STATEMENT BELOW THAT PUTS ALL FREQS TOGETHER */
   %let SETEVT =  %str(EVT_N (keep= &COLVAR &SUBVAR frequency &GRPVAR &EVTVAR));

%end;

/************************************************************************/
/* PULL ALL OF THE VARIOUS TOTALS TOGETHER 				*/
/* OVERALL COLVAR IS DEFINED AS "TOTAL" IN rptcol WHEN COLVAR missing	*/
/* OVERALL SUBVAR IS DEFINED AS "All"					*/ 
/************************************************************************/
   data soclose;
       set &SETALL
           &SETGRP
           &SETEVT   
   	   end = eof
       ;

/* ENTER DENOM VARIABLES */
     length &DENOMLIST 8.;
     %do D=1 %to &CLISTMAX;
        &&FLAGDENOM&D;
        label &&LBLDENOM&D;
     %end;

/* THESE RECS ARE TOTALS */
/* DENOM VARIABLE LOGIC MUST STAY WITHIN COLVTYPE CODE */
       denom&COLTOTVAL 	     = &MACCOMTOT;* sum(&DENOMLISTCOM);
       %let DENOMTOTAL 	     = denom&COLTOTVAL;
       label denom&COLTOTVAL = "DENOM FOR ALL COLVAR VALUES";

/* ADD VARIABLES WITH VALID VARIABLE VALUES FOR COLVAR VALUES */
/* MAKE percent AND pctdenom VARS */
       length colvnm $&CVNMTOP.. percent 8. ;
   %do E=1 %to &CLISTMAX;
     %if &E = 1 %then %do;
       if      strip(vvalue(&COLVAR)) = "&&THISCVAL&E" then do;
     %end;
     %else %do;
       else if strip(vvalue(&COLVAR)) = "&&THISCVAL&E" then do;
     %end;
           	colvnm    = "%nrbquote(&&VARCNAME&E)";
           	percent   = 100*(frequency/denom&&VARCNAME&E);
           	pctdenom  = denom&&VARCNAME&E;
       end;
   %end;    
/* END OF &E TO &CLISTMAX LOOP */
       else do;
	   	colvnm    = strip(vvalue(&COLVAR));
	   	percent   = 100*(frequency/&DENOMTOTAL);
	   	pctdenom  = &DENOMTOTAL;
       end;
    
       label 	colvnm    = "COLVAR VALUE USED IN DENOM"
		percent   = "PERCENT = 100*(frequency/denom<colvnm>)"
		pctdenom  = "DENOM VALUE USED FOR EACH percent CALCULATION"
		;

/* VAR CREATION FOR SIMPLE REPORTING */
        rptpct = '('||strip(put(percent,&PCTFMT))||'%)';
        label   rptpct    = "FORMATTED PERCENT VALUE FOR REPORT"
                ;        

        /* CREATE grpev1col VAR WITH INDENTATION FOR EVENT ROWS */
       length grpev1col $&GRPEVFTW;
            if missing(&GRPVAR) and 
               missing(&EVTVAR) then  grpev1col = "Overall";
       else if missing(&EVTVAR) then  grpev1col = strip(vvalue(&GRPVAR));
       else                           grpev1col = cat('    ',strip(vvalue(&EVTVAR)));

       label grpev1col = "%upcase(&GRPVAR) and %upcase(&EVTVAR) values in one variable"
         ;

/* ADD rptgrp AND rptevt VARS FOR FLAGRPT AUTOMATION */
       length rptgrp $&GRPVARFTW;
  	      rptgrp = strip(vvalue(&GRPVAR));
       label  rptgrp = "RPTGRP: FLAGSUM &GRPVAR VARIABLE FOR AUTOMATION OF FLAGRPT";

       length rptevt $&EVTVARFTW;
              rptevt = strip(vvalue(&EVTVAR));
       label  rptevt = "RPTEVT: FLAGSUM &EVTVAR VARIABLE FOR AUTOMATION OF FLAGRPT";

/* MAKE REPORTING VARS FOR COLVAR AND SUBVAR */
/* NEED INCREASED LENGTH                     */
/* ADDED &TOTLEN TO %eval() BELOW */
/* 	TOTLEN = LENGTH OF denomTOTAL value	 */
   %let RPTCOLW = %eval(&COLVARFTW.+&TOTLEN.+10);
       length rptcol $&RPTCOLW..;
       label  rptcol = "REPORT READY &COLVAR WITH/WITHOUT (N=) VALUE";

/* IF PCTMODE = 2 OR 9, NO DENOMS IN THE TITLE 	*/
/* OTHERWISE DENOMS ARE IN THE TITLE AS "N="	*/
   %if &PCTMODE = 2 or 
       &PCTMODE = 9 %then %do;
       if missing(&COLVAR) then rptcol = "&COLTOTVAL";
       else		        rptcol = strip(vvalue(&COLVAR));
   %end;
   %else %do;
       if missing(&COLVAR) then rptcol = "&COLTOTVAL"||'$(N='||strip(pctdenom)||')';
       else                     rptcol = strip(vvalue(&COLVAR))||'$(N='||strip(pctdenom)||')';
   %end;

   %if %length(&SUBVAR) > 0 %then %do;
       length rptsub $&SUBVARFTW..;
       label  rptsub = "REPORT READY &SUBVAR";

       if missing(&SUBVAR) then rptsub = "&SUBTOTVAL";
       else                     rptsub = strip(vvalue(&SUBVAR));
   %end;

/* HELPS TO ALIGN FREQS IN .lst AND MAKE IT CHARACTER FOR REPORTING */
       label rptfrq = "REPORT READY frequency";	
             rptfrq = put(frequency,3. -r);

/* GET WIDTH OF freq AND pct VARS FOR rptme VAR IN DATA STEP BELOW */
       call symput("PCTRPTW",strip(vformatw(rptpct)));
       call symput("FRQRPTW",strip(vformatw(rptfrq)));
   run;

/* CREATE MACRO VAR THAT ADDS LENGTH OF FREQ AND PCT VAR 	*/
/* ADDED 1 TO EQUATION FOR SPACE BETWEEN FREQ AND PCT VALUES 	*/
%let   RPTMEW = %eval(&PCTRPTW+&FRQRPTW+1);

/* NEEDED LENGTH OF rptpct VALUES TO DETERMINE LENGTH OF rptme  */
/* AND LOGIC WAS COMPLICATED, SO USED ANOTHER DATA STEP         */
   data &OUTDATA;
      set soclose;

/* rptme CODE FOR FREQS AND PCTS TO SHOW IN REPORT */
/***************************************************/
      %if          &PCTMODE = 0 or
		   &PCTMODE = 9 %then %do;
	length rptme $&FRQRPTW..;
               rptme     = rptfrq;
	label  rptme     = "N";
      %end;
      %else %if    &PCTMODE = 2 or 
		   &PCTMODE = 8 %then %do;
	length rptme $&PCTRPTW..;
               rptme 	 = rptpct;
	label  rptme     = "(%)";
      %end;
      %else %do;
	length rptme $&RPTMEW..;
	label  rptme     = "N (%)";

         %if       &PCTMODE = 3  %then %do;
                rptme = rptfrq||' '||rptpct;
         %end;
	 %else %do;
/* BELOW ARE THE CODITIONS FOR THE rptme CODE CHANGE, 	*/
/* BUT FINAL VALUE ASSIGNMENT CODE IS THE SAME.		*/
            %if       &PCTMODE = 1  %then %do;
               if missing(&EVTVAR) then
            %end;
            %else %if &PCTMODE = 4  %then %do;
               if missing(&EVTVAR) and missing(&COLVAR) then
	    %end;
            %else %if &PCTMODE = 5  %then %do;
	       if missing(&COLVAR) then
	    %end;
            %else %if &PCTMODE = 6  %then %do;
               if missing(&EVTVAR) and missing(&SUBVAR) then
	    %end;
            %else %if &PCTMODE = 7  %then %do;
               if missing(&SUBVAR) then
	    %end;
            %else %if &PCTMODE = 10 %then %do;
               if missing(&EVTVAR) or missing(&SUBVAR) then
	    %end;
/* SAME CODE TO ASSIGN VALUES */
               	     rptme  = rptfrq||' '||rptpct;
               else  rptme  = rptfrq;
	 %end;
      %end;

   run;

   /* KEEP TO SORT BY ALPHA ORDER FOR GRPVAR AND THEN EVTVAR */
   proc sort data = &OUTDATA;
       by &GRPVAR &EVTVAR;
   run;

/***************/
/* MAKE TITLES */
/***************/
      title  "THE DATASET PRINTED BELOW IS NAMED &OUTDATA."; 
      title2 "THIS DATASET IS SORTED BY &GRPVAR AND &EVTVAR FOR EASY VIEWING.";
%if &LSTMORE = Y %then %do;

      title3 "SET LSTMORE = N, FOR _LESS_ INFORMATION IN THE .lst.";
      title4 "VARIABLES STARTING WITH rpt ARE INTENDED FOR REPORTING.";
      title5 "ALONG WITH EITHER grpev1col OR THE COMBINATION OF &GRPVAR AND &EVTVAR.."; 
      title6 "PRINT SHOWS AE TOTALS FOR FLAGS:  &ALLFLG &GRPFLG &EVTFLG";
   %if %length(&COLWHERE) > 0 %then %do;
      title7 "WITH COLWHERE SET TO:  &COLWHERE";
   %end;
%end;
%else %do;
      title3 "THE VARIABLES &GRPVAR AND &EVTVAR ARE NOT SHOWN.";
      title4 "SET LSTMORE = Y, FOR _MORE_ INFORMATION IN THE .lst";
      title5 "ONLY VARIABLES INTENDED FOR REPORTING ARE SHOWN.";
      title6 "SHOWS AE TOTALS FOR FLAGS:  &ALLFLG &GRPFLG &EVTFLG";
   %if %length(&COLWHERE) > 0 %then %do;
      title7 "WITH COLWHERE SET TO:  &COLWHERE";
   %end;
%end;

/* OUTPUT */
   proc print    data= &OUTDATA;
      %if &LSTMORE ^= Y %then %do; 
         var grpev1col rptcol 
             %if %length(&SUBVAR) %then %do;
	        rptsub
	     %end;
	     rptfrq pctdenom rptme; 
      %end;
   run;

   title  "SHOWS ALL VARIALBLES IN &OUTDATA!!";
%if &LSTMORE ^= Y %then %do;
   title2 "NOT ALL VARIABLES ARE IN PROC PRINT ABOVE.";
%end;

   proc contents data= &OUTDATA;
   run;

/* SET missing OPTION BACK TO USER SETTING BEFORE flagsum MACRO */
options missing="&USERMISS";

%mend  flagsum;

